"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var request = require("request-promise");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var baseTracker_1 = require("../baseTracker");
var utilities_1 = require("../utilities");
var trackingBaseUrl = 'https://www.fedex.com/trackingCal/track';
var Fedex = /** @class */ (function (_super) {
    __extends(Fedex, _super);
    function Fedex(trackingNumber) {
        var _this = _super.call(this, trackingNumber) || this;
        _this.carrierName = 'Fedex';
        return _this;
    }
    Fedex.prototype.isTrackingNumberFromCarrier = function () {
        return this.getPackageInformationFromCarrier().pipe(operators_1.map(function (response) {
            /* This part is still fuzzy. We assume that a tracking number from fedex
             is valid if there is a keyStatusCD in the response*/
            response = JSON.parse(response);
            if (response.TrackPackagesResponse &&
                response.TrackPackagesResponse.successful &&
                response.TrackPackagesResponse.packageList &&
                response.TrackPackagesResponse.packageList.length !== 0 &&
                response.TrackPackagesResponse.packageList[0].keyStatusCD &&
                response.TrackPackagesResponse.packageList[0].keyStatusCD.length > 0) {
                return true;
            }
            else {
                return false;
            }
        }), operators_1.catchError(utilities_1.handleError("isTrackingNumberFromCarrier Fedex", false)));
    };
    Fedex.prototype.getPackageInformationFromCarrier = function () {
        var data = {
            TrackPackagesRequest: {
                trackingInfoList: [
                    {
                        trackNumberInfo: {
                            trackingCarrier: '',
                            trackingNumber: "" + this.trackingNumber,
                            trackingQualifier: '',
                        },
                    },
                ],
            },
        };
        var options = {
            form: {
                action: 'trackpackages',
                data: JSON.stringify(data),
                format: 'json',
            },
            method: 'POST',
            uri: trackingBaseUrl,
        };
        // construct observable from promise
        return rxjs_1.from(request(options));
    };
    /**
     * Converts Fedex response to ITrackingInfo interface
     * @param response
     */
    Fedex.prototype.resolveCarrierResponseToITrackingInfo = function (response) {
        // response needs to be parsed
        response = JSON.parse(response);
        /**
         * Throw error if unable to handle package response
         */
        if (!response.TrackPackagesResponse ||
            !response.TrackPackagesResponse.packageList ||
            response.TrackPackagesResponse.packageList.length === 0) {
            throw new Error("Error when handling response from Fedex");
        }
        // For the moment we only handle the first package of the list.
        // TODO Find better handling implementation
        var firstPackage = response.TrackPackagesResponse.packageList[0];
        var label = firstPackage.keyStatus;
        var percentage = 0;
        /* Fedex has 4 statuses */
        switch (label) {
            case 'Initiated':
                percentage = 25;
                break;
            case 'Picked up':
                percentage = 50;
                break;
            case 'In transit': // keyStatusCD "LO"
                percentage = 75;
                break;
            case 'Delivered': // TODO MAKE SURE keyStatusCD "DL"
                percentage = 100;
                break;
            default:
                percentage = 0;
        }
        return {
            carrierName: this.carrierName,
            detailsLink: this.getDetailsLink(),
            statusLabel: label,
            statusPercentage: percentage,
        };
    };
    Fedex.prototype.getDetailsLink = function () {
        return "https://www.fedex.com/apps/fedextrack/?tracknumbers=" + this.trackingNumber;
    };
    return Fedex;
}(baseTracker_1.PackageTracker));
exports.Fedex = Fedex;
