"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var request = require("request-promise");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var baseTracker_1 = require("../baseTracker");
var utilities_1 = require("../utilities");
var trackingBaseUrl = 'https://www.ups.com/track/api/Track/GetStatus?loc=en_US';
var UPS = /** @class */ (function (_super) {
    __extends(UPS, _super);
    function UPS(trackingNumber) {
        var _this = _super.call(this, trackingNumber) || this;
        _this.carrierName = 'UPS';
        return _this;
    }
    UPS.prototype.isTrackingNumberFromCarrier = function () {
        return this.getPackageInformationFromCarrier().pipe(operators_1.map(function (response) {
            if (response &&
                +response.statusCode === 200 &&
                response.trackDetails &&
                response.trackDetails.length > 0 &&
                +response.trackDetails[0].errorCode === 0) {
                return true;
            }
            else {
                return false;
            }
        }), operators_1.catchError(utilities_1.handleError("isTrackingNumberFromCarrier UPS", false)));
    };
    UPS.prototype.getPackageInformationFromCarrier = function () {
        var options = {
            body: {
                Locale: 'en_US',
                TrackingNumber: ["" + this.trackingNumber],
            },
            headers: {
                Accept: 'application/json',
                'Content-Type': 'application/json',
                'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/71.0.3578.98 Safari/537.36',
            },
            json: true,
            method: 'POST',
            uri: trackingBaseUrl,
        };
        // construct observable from promise
        return rxjs_1.from(request(options));
    };
    /**
     * Convert UPS response to ITrackingInfo interface
     * @param response
     */
    UPS.prototype.resolveCarrierResponseToITrackingInfo = function (response) {
        var label = '';
        var percentage = 0;
        var details = response.trackDetails;
        if (!details) {
            throw new Error("Error when handling response from UPS");
        }
        if (details.length > 0) {
            label = details[0].packageStatus;
            percentage = details[0].progressBarPercentage;
            if (!percentage) {
                switch (details[0].progressBarType) {
                    case 'OutForDelivery':
                        percentage = 90;
                        break;
                }
            }
        }
        return {
            carrierName: this.carrierName,
            detailsLink: this.getDetailsLink(),
            statusLabel: label,
            statusPercentage: +percentage,
        };
    };
    UPS.prototype.getDetailsLink = function () {
        return "https://www.ups.com/track?loc=en_US&tracknum=" + this.trackingNumber + "&requester=WT/trackdetails";
    };
    return UPS;
}(baseTracker_1.PackageTracker));
exports.UPS = UPS;
