"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var fedex_1 = require("./carriers/fedex");
var ups_1 = require("./carriers/ups");
/**
 * Guesses the Carrier depending on the tracking number
 * Returns an observable that will resolve with the corresponding carrier
 * or an error
 * @param trackingNumber
 */
var GuessCarrier = function (trackingNumber) {
    // create all the available trackers
    var upsTracker = new ups_1.UPS(trackingNumber);
    var fedexTracker = new fedex_1.Fedex(trackingNumber);
    // group them in an array
    var trackers = [upsTracker, fedexTracker];
    // create array of observable from the trackers array
    var trackerObservables = trackers.map(function (tracker) {
        return tracker.isTrackingNumberFromCarrier();
    });
    return rxjs_1.forkJoin(trackerObservables).pipe(operators_1.map(function (result) {
        var firstMatchingTrackerIndex = result.indexOf(true);
        if (firstMatchingTrackerIndex !== -1) {
            return trackers[firstMatchingTrackerIndex];
        }
        throw new Error('No carrier found for this tracking number');
    }));
};
exports.Track = function (trackingNumber, callback) {
    GuessCarrier(trackingNumber).subscribe(function (carrier) {
        carrier.track().subscribe(function (infos) {
            callback(infos, null);
        }, function (err) {
            callback({}, err);
        });
    }, function (err) {
        callback({}, err);
    });
};
// Track('479242596176', (infos) => {
//   console.log('infos', infos);
// });
