"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var rxjs_1 = require("rxjs");
/**
 * Handle Http operation that failed.
 * Let the app continue.
 * The following errorHandler() is shared by many ProfileService methods so it's generalized
 * to meet their different needs.
 * Instead of handling the error directly, it returns an error handler function to catchError
 * that it has configured with both the name of the operation that failed and a safe return
 * value.
 * @param operation - name of the operation that failed
 * @param result - optional value to return as the observable result
 */
function handleError(operation, result) {
    if (operation === void 0) { operation = 'operation'; }
    return function (error) {
        // console.error(`${operation} failed: ${error.message}`); // log the error
        // Let the app keep running by returning an empty result.
        return rxjs_1.of(result);
    };
}
exports.handleError = handleError;
